/*	IEEE802_11.h
 *	
 *	Copyright 2011 atzeton <atzeton@nullsecurity.net>
 *	
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *	
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *	
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *	MA 02110-1301, USA.
 */




/* radiotap.h
 * radiotap header has been reverse engineered. See official releases for better information.
 */

#ifndef HDR_80211
#define HDR_80211

#include <inttypes.h>

#define IEEE80211_ASSOCIATION_REQUEST 		(0x00)
#define IEEE80211_ASSOCIATION_RESPONSE 		(0x01)
#define IEEE80211_REASSOCIATION_REQUEST  	(0x02)
#define IEEE80211_REASSOCIATION_RESPONSE 	(0x03)
#define IEEE80211_PROBE_REQUEST 			(0x04)
#define IEEE80211_PROBE_RESPONSE 			(0x05)
#define IEEE80211_MEASUREMENT_PILOT 		(0x06)
#define IEEE80211_RESERVED 					(0x07) 
#define IEEE80211_BEACON 					(0x08)
#define IEEE80211_ATIM 						(0x09)
#define IEEE80211_DISASSOCIATION 			(0x0a)
#define IEEE80211_AUTHENTICATION			(0x0b)
#define IEEE80211_DEAUTHENTICATION			(0x0c)
#define IEEE80211_ACTION 					(0x0d)
#define IEEE80211_ACTION_NOACK 				(0x0e)

#define IEEE80211_CTS		 				(0x1c)
#define IEEE80211_ACK		 				(0x1d)
#define IEEE80211_DATA		 				(0x20)
#define IEEE80211_QOS_DATA	 				(0x28)
#define IEEE80211_NULL		 				(0x24)

/* tagged parameters of the Beacon wireless management frame | INCOMPLETE */
#define IEEE80211_BEACON_TAGGED_SSID		(0x00)
#define IEEE80211_BEACON_TAGGED_RATES		(0x01)
#define IEEE80211_BEACON_TAGGED_CHANNEL		(0x03)

/* used as a preamble !(+ duration) */
#define FCTRL_SIZE (2)
typedef struct IEEE80211_FCTRL {
	unsigned version:2;
	unsigned type:2;
	unsigned subtype:4;
	unsigned to_ds:1;
	unsigned from_ds:1;
	unsigned mf:1;
	unsigned retry:1;
	unsigned pwr:1;
	unsigned moredata:1;
	unsigned protected:1;
	unsigned order:1;
} IEEE80211_FCTRL_t;

typedef struct IEEE80211_MAC {
	uint16_t fctrl;
	uint16_t drtn;
	uint8_t dst[6];
	uint8_t src[6];
	uint8_t bssid[6];
	uint16_t seq;
} IEEE80211_MAC_t;

typedef struct IEEE80211_BEACON_t {
	uint16_t fctrl;
	uint16_t drtn;
	uint8_t dst[6];
	uint8_t src[6];
	uint8_t bssid[6];
	uint16_t seq;
} IEEE80211_BEACON_t;
	
typedef struct IEEE80211_BEACON_FIXEDPARAM {
	unsigned timestamp1:32;
	unsigned timestamp2:32;
	
	unsigned beacon_interval:16;
	unsigned ess_ap:1;
	unsigned ibss_stat:1;
	unsigned cfp_part:2;
	unsigned wep:1;
	unsigned short_preamble:1;
	unsigned pbcc:1;
	unsigned chan_agi:1;
	unsigned spec_man:1;
	unsigned cfp_part2:1;
	unsigned sslottime:1;
	unsigned apsd:1;
	unsigned dsss_ofdm:1;
	unsigned del_bl_ack:1;
	unsigned imm_bl_ack:1;
} IEEE80211_BEACON_FIXEDPARAM_t;


typedef struct IEEE80211_PROBE_REQUEST_t {
	uint16_t duration;	
	uint8_t dst[6];
	uint8_t src[6];
	uint8_t bssid[6];
	uint16_t seq;
} IEEE80211_PROBE_REQUEST_t;

typedef struct IEEE80211_PROBE_RESPONSE_t {
	uint16_t duration;	
	uint8_t dst[6];
	uint8_t src[6];
	uint8_t bssid[6];
	uint16_t seq;		
} IEEE80211_PROBE_RESPONSE_t;


/* ToDS */
typedef struct IEEE80211_DATA_t {
	uint16_t duration;	
	uint8_t bssid[6];
	uint8_t src[6];
	uint8_t dst[6];
	uint16_t seq;
} IEEE80211_DATA_t;

typedef struct IEEE80211_DATA2_t {
	uint16_t duration;	
	uint8_t dst[6];
	uint8_t bssid[6];
	uint8_t src[6];
	uint16_t seq;
} IEEE80211_DATA2_t;

typedef struct IEEE80211_CTS_t {
	uint16_t duration;	
	uint8_t dst[6];
} IEEE80211_CTS_t;
	
typedef struct IEEE80211_AUTH_t {
	uint16_t duration;	
	uint8_t dst[6];
	uint8_t src[6];
	uint8_t bssid[6];
	uint16_t seq;
} IEEE80211_AUTH_t;

typedef struct IEEE80211_AUTH_FIXED_t {
	uint16_t algorithm;
	uint16_t seq;
	uint16_t code;
} IEEE80211_AUTH_FIXED_t;



typedef struct IEEE80211_DEAUTH_t {
	uint16_t duration;	
	uint8_t dst[6];
	uint8_t src[6];
	uint8_t bssid[6];
	uint16_t seq;
} IEEE80211_DEAUTH_t;

typedef struct IEEE80211_NULL_t {
	uint8_t recv[6];
	uint8_t tansm[6];
	uint8_t dst[6];
	uint16_t seq;
	uint8_t src[6];
} IEEE80211_NULL_t;

typedef struct IEEE80211_ACK_t {
	uint16_t duration;	
	uint8_t dst[6];
} IEEE80211_ACK_t;



#endif

