/*	hawk.h
 *	
 *	Copyright 2011 atzeton <atzeton@nullsecurity.net>
 *	
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *	
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *	
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *	MA 02110-1301, USA.
 */



#ifndef HDR_HAWK
#define HDR_HAWK

#include <inttypes.h>
#include <pcap.h>
#include <linux/wireless.h>
#include "radiotap.h"

#define VERSION "0.3.2 BETA"
#define CODENAME "_rebhuhn"

#define TRUE			  	(0 == 0)
#define FALSE		   	( !TRUE) 		/* well, false is not true xD */

#define MAC_ZERO_ADDR	 	"\x00\x00\x00\x00\x00\x00"
#define MAC_BROADCAST_ADDR	"\xff\xff\xff\xff\xff\xff"

/* channel walkthrough/hopping timeout */
#define CHWALKTHROUGH	  	(500000)
#define PACKET_TIMEOUT	 	(5000)

/* channels to use, you may change them (e.g. add 14) */
uint8_t channels[] =  		{1,2,3,4,5,6,7,8,9,10,11,12,13};

/* hwk modes as specified using e.g. --scan --deauth */
#define HWK_MODE_SCAN	   (0)
#define HWK_MODE_DEAUTH	 (1)
#define HWK_MODE_AUTH	   (2)
#define HWK_MODE_FOCUS	  (3)
#define HWK_MODE_BEACON	 (4)
#define HWK_MODE_TESTINJECT (5)
#define HWK_MODE_FUZZPROBES (6)
#define HWK_MODE_666_9	  (9)

typedef struct client_elem {
	uint8_t addr[6];
	struct client_elem *next;
} client_t;

typedef struct netelem {
	uint8_t bssid[6];
	int8_t channel;
	uint16_t lseq;
	client_t *first_client;
	client_t *last_client;
	struct netelem *next; /* next bssid */
} bssid_t;



typedef struct OPTS_t {
	pcap_t  *pcapd;
	char	 pcap_errbuf[PCAP_ERRBUF_SIZE]; 
	
	char	*ssid;
	
	uint8_t  mode;
	int8_t   dev_auto;
	char	 device[IFNAMSIZ];
	uint32_t dlt;
	int32_t  sockfd;
	int16_t  channel;
	uint8_t  eX7r3m3_fl00d; /* only for REAL 1337s */
	int8_t   injection_works; 
	uint8_t  bssid[6];
	uint8_t  dstmac[6];
	uint8_t  clmac[6];
	
	int8_t   channel_changed;
	uint8_t *mac_fake_addr;

	uint16_t current_seq;
	int8_t   ssi_signal;

	uint32_t pkt_sum;
	uint32_t pkt_correct;
	uint32_t pkt_beacons;
	uint32_t pkt_data;
	uint32_t pkt_ack;
	uint32_t pkt_injected;
	uint32_t twait;

	int8_t   scan_done;
	
	bssid_t *first_bssid;
	bssid_t *last_bssid;

} OPTS_t;


int8_t	  attack_beacon_flood(OPTS_t *opts);
void 	   *ch_walkthrough(void *arg);
uint16_t 	radiotap_get_total_length(RADIOTAP_CONTROL_t *rmanage);
void 		print_mac_addr(uint8_t *addr);
int8_t 		radiotap_parse(const uint8_t *pkt, RADIOTAP_CONTROL_t *rmanage);
uint16_t	seq2seq(uint16_t seq);
void 	   *seqalyzer(void *arg);
void 		set_channel(char *dev, int16_t channel);
int8_t 		wifi_freq2chan(uint16_t freq);
void	   *mode9_catch_data(void *arg);
void	   *mode9_killall(void *arg);
int8_t	  bssid_has_client( uint8_t *clmac, OPTS_t *opts);
int8_t	  bssid_is_in_list(uint8_t *bssid, bssid_t *first);
void		show_networks(OPTS_t *opts);
int8_t	  focus(OPTS_t *opts);
uint8_t	*calc_mac_fake_addr(void);
void	   *inject_auth(void *arg);
void	   *inject_deauth(void *arg); 
int8_t	  test_inject(OPTS_t *opts);
int16_t	 get_channel(OPTS_t *opts);
int8_t	  get_beacon(OPTS_t *opts);
void	   *ch_perm_hop(void *arg);
int8_t	  optget(int argc, char** argv, OPTS_t *opts);
void		print_timestamp();
void		clear_line();
RADIOTAP_CONTROL_t *radiotap_init(void);
#endif

